#! /bin/sh
#
# Author: Juergen Vigna <juergen.vigna@wuerth-phoenix.com>
# Version: 1.0.0
# Licence: GPLV2
# Params: -H host [-F <from-time>] [-T <to-time>]
# Description:
#   This scripts sends a host performance report to the email recipients specified"
#

TMPFILE="/var/tmp/host_perf_report$$.tmp"
DEBUGFILE="/var/tmp/host_perf_report$$.debug"
#EMAIL=sistemas@wurth.es,xmasats@wurth.es,dguisado@wurth.es,jmanyas@wurth.es,jdiaz@wurth.es,apalop@wurth.es
#EMAIL=juergen.vigna@wuerth-phoenix.com,nfontani@wurth.es
EMAIL=juergen.vigna@wuerth-phoenix.com

trap 'rm -f $TMPFILE $DEBUGFILE; exit 10' 1 2 15
trap 'rm -f $TMPFILE $DEBUGFILE' 0

VERSION="1.0.0"
PROGNAME=`basename $0`
PROGPATH=`echo $0 | /bin/sed -e 's,[\\/][^\\/][^\\/]*$,,'`

PATH=$PATH:/bin:/usr/bin:/usr/local/bin
export PATH

TT=`type -P get_rrd_data_average.sh 2>/dev/null`
if [ -f /opt/neteye/bin/get_rrd_data_average.sh ]
then
	RRD_VALUES_CMD=/opt/neteye/bin/get_rrd_data_average.sh
elif [ -f $PROGPATH/get_rrd_data_average.sh ]
then
	RRD_VALUES_CMD=$PROGPATH/get_rrd_data_average.sh
elif [ -n "$TT" -a -f "$TT" ]
then
	RRD_VALUES_CMD="$TT"
else
	echo "ERROR: helper script get_rrd_data_average.sh not found!"
	exit 3
fi

print_header()  {
        echo "$PROGNAME - $VERSION"
        echo ""
        echo "This scripts sends a host performance report to the email recipients specified"
        echo ""

}

print_usage() {
        echo "Usage: $PROGNAME [-D] [-F <from-time>] [-T <to-time>] -H hostname"
	echo ""
        echo "       -h, --help    : this help"
        echo "       -V, -version  : program version"
        echo "       -D            : output debug informations"
        echo "       -H            : Hostname as defined inside Nagios Configuration"
        echo "       -F            : from time in the format HH:MM:SS (default: 18:00:00)"
        echo "       -T            : to time in the format HH:MM:SS (default: 23:59:59)"
        echo
        echo "The time format is HH (00-23), MM (00-59), SS (00-59)"
        echo
}

print_help() {
        print_header
        print_usage
        echo ""
}

# --------------------------------------------------------------------------- 
# INITIALIZE VARIABLES
# --------------------------------------------------------------------------- 

DEBUG=0
HOST=""
FROMTIME="18:00:00"
TOTIME="23:59:59"
CPUSERVICE="*SAR_CPU-USED"
CPUPROCSSERVICE="*SAR_Cola_de_procesos_en_CPU"
WIOSERVICE="*SAR_CPU-IOWAIT"
PROCSSERVICE="*Total_Processes"
SWAPSERVICE="*SAR_Swaping"
MEMSERVICE="*Memory_Usage"
DIOSERVICE="*SAR_Disk_IO"

# --------------------------------------------------------------------------- 
# OPTION HANDLING
# --------------------------------------------------------------------------- 

while [ -n "$1" ]
do
    opt=`echo "$1" | cut -d= -f1`
    optarg=`echo "$1" | cut -d= -f2-`
    if [ -z "$optarg" ]
    then
        echo "Argument not specified for extended option $1"
        exit 3
    fi
    if [ "$optarg" = "$opt" ]
    then
        optarg="$2"
    fi
    case "$opt" in
        --help)
            print_help
            exit $STATE_OK
            ;;
        -h)
            print_help
            exit $STATE_OK
            ;;
        --version)
            print_revision $PROGNAME $VERSION
            exit $STATE_OK
            ;;
        -V)
            print_revision $PROGNAME $VERSION
            exit $STATE_OK
            ;;
        --host | -H)
            HOST=$optarg
            if [ "$optarg" = "$2" ]
            then
                shift
            fi
            ;;
        --from-time | -F)
            FROMTIME=$optarg
            if [ "$optarg" = "$2" ]
            then
                shift
            fi
            ;;
        --to-time | -T)
            TOTIME=$optarg
            if [ "$optarg" = "$2" ]
            then
                shift
            fi
            ;;
        --debug | -D)
            DEBUG=1
            ;;
        *)
            print_usage
            echo "Unknown argument: $1"
            exit 3
            ;;
    esac
    shift
done

if [ -z "$HOST" ]
then
    print_usage
    echo "Missing HOST parameter"
    exit 3
fi

if [ -z "$FROMTIME" -o -z "$TOTIME" ]
then
	print_usage
	echo "Missing from/to time parameter"
	exit 3
fi
 
send_html_email() {

DE=$1
PARA=$2
ASUNTO="Tendencia de rendimiento de $4"
FILE=$3
MAQUINA=$4
INTERVALO=$5
FECHA_DIA=$6
FECHA_ANT=$7
FECHA_DSA=$8

(
 echo "Subject: $ASUNTO"
 echo "To: $PARA"
 echo "From: $DE"
 echo "MIME-Version: 1.0"
 echo "Content-Type: text/html"
 echo "Content-Disposition: inline"

 cat $FILE|awk -v MAQUINA=$MAQUINA -v INTERVALO="$INTERVALO" -v FECHA_DIA=$FECHA_DIA -v FECHA_ANT=$FECHA_ANT -v FECHA_DSA=$FECHA_DSA -F"|" '
 BEGIN {

   print "<HTML>"
   print "<BODY>"
   print "<LEFT><TABLE BORDER=\"1\" CELLSPACING=\"0\" CELLPADDING=\"2\" WIDTH=350 >"
   print "<TR>"
   print "<TH COLSPAN=2 BGCOLOR=#000080 WIDTH=150><FONT SIZE=2><FONT FACE=Arial COLOR=white>Tabla de tendencia de la media de los principales parametros del sistema.</FONT></TH>"
   print "</TR>"

   print "<TR>"
   print "<TD BGCOLOR=#E0E0E0 ALIGN=LEFT><FONT SIZE=2><FONT FACE=Arial>Maquina</FONT></TD>"
   print "<TD BGCOLOR=#E0E0E0 ALIGN=LEFT><FONT SIZE=2><FONT FACE=Arial>"MAQUINA"</FONT></TD>"
   print "</TR>"

   print "<TR>"
   print "<TD BGCOLOR=#E0E0E0 ALIGN=LEFT><FONT SIZE=2><FONT FACE=Arial>Intervalo de medida</FONT></TD>"
   print "<TD BGCOLOR=#E0E0E0 ALIGN=LEFT><FONT SIZE=2><FONT FACE=Arial>"INTERVALO"</FONT></TD>"
   print "</TR>"

   print "</table> "

   print "<LEFT><TABLE BORDER=\"1\" CELLSPACING=\"0\" CELLPADDING=\"2\" WIDTH=900 >"
   print "<TR>"
   print "<TH BGCOLOR=\"#000080\" WIDTH=200><FONT SIZE=2><FONT FACE=\"Arial, Helvetica\" COLOR=\"white\">Metrica</FONT></TH>"
   print "<TH BGCOLOR=\"#000080\" WIDTH=70><FONT SIZE=2><FONT FACE=\"Arial, Helvetica\" COLOR=\"white\">Ayer ("FECHA_DIA")</FONT></TH>"
   print "<TH BGCOLOR=\"#000080\" WIDTH=70><FONT SIZE=2><FONT FACE=\"Arial, Helvetica\" COLOR=\"white\">Dia anterior ("FECHA_ANT")</FONT></TH>"
   print "<TH BGCOLOR=\"#000080\" WIDTH=70><FONT SIZE=2><FONT FACE=\"Arial, Helvetica\" COLOR=\"white\">Dia sem. anterior ("FECHA_DSA")</FONT></TH>"
   print "<TH BGCOLOR=\"#000080\" WIDTH=70><FONT SIZE=2><FONT FACE=\"Arial, Helvetica\" COLOR=\"white\">Ultimos 7 dias</FONT></TH>"
   print "<TH BGCOLOR=\"#000080\" WIDTH=70><FONT SIZE=2><FONT FACE=\"Arial, Helvetica\" COLOR=\"white\">Ultimos 30 dias</FONT></TH>"
   print "</TR>"
  }
         {
         print "<TR>"
         print "<TD BGCOLOR=\"#E0E0E0\" ALIGN=\"LEFT\"><FONT SIZE=2><FONT FACE=Arial>"$1"</FONT></TD>"
         if ($2 > $3) ALERTA="red"; else ALERTA="green";
         print "<TD BGCOLOR=\"#E0E0E0\" ALIGN=\"RIGHT\"><FONT SIZE=2><FONT FACE=Arial COLOR="ALERTA">"$2"</FONT></TD>"
         if ($3 > $4) ALERTA="red"; else ALERTA="green";
         print "<TD BGCOLOR=\"#E0E0E0\" ALIGN=\"RIGHT\"><FONT SIZE=2><FONT FACE=Arial COLOR="ALERTA">"$3"</FONT></TD>"
         if ($4 > $5) ALERTA="red"; else ALERTA="green";
         print "<TD BGCOLOR=\"#E0E0E0\" ALIGN=\"RIGHT\"><FONT SIZE=2><FONT FACE=Arial COLOR="ALERTA">"$4"</FONT></TD>"
         if ($5 > $6) ALERTA="red"; else ALERTA="green";
         print "<TD BGCOLOR=\"#E0E0E0\" ALIGN=\"RIGHT\"><FONT SIZE=2><FONT FACE=Arial COLOR="ALERTA">"$5"</FONT></TD>"
         if ($6 > $7) ALERTA="red"; else ALERTA="green";
         print "<TD BGCOLOR=\"#E0E0E0\" ALIGN=\"RIGHT\"><FONT SIZE=2><FONT FACE=Arial COLOR="ALERTA">"$6"</FONT></TD>"
         print "</TR>"
         }
 END {
   print "</TABLE></CENTER>"
   print "<BR>"
   print "<FONT SIZE=2><FONT FACE=Arial> LEYENDA:<BR>"
   print "<FONT SIZE=2><FONT FACE=Arial> Todos los datos son una media de los valores recogidos en el intervalo de tiempo indicado, por tanto puede que en<BR>"
   print "<FONT SIZE=2><FONT FACE=Arial> momentos determinados los picos superen esta media ampliamente. Esto es una tabla de tendencias, y pretende indicar<BR>"
   print "<FONT SIZE=2><FONT FACE=Arial> hacia donde evoluciona un sistema.<BR>"
   print "<FONT SIZE=2><FONT FACE=Arial> - Los datos de ayer, son la media de los valores recogidos en el intervalo de tiempo.<BR>"
   print "<FONT SIZE=2><FONT FACE=Arial> - Los datos del dia anterior, sirven para comparar dos dias consecutivos.<BR>"
   print "<FONT SIZE=2><FONT FACE=Arial> - Los datos del dia semana anterior, se refieren al mismo dia, pero hace una semana.<BR>"
   print "<FONT SIZE=2><FONT FACE=Arial> - El resto de datos se han tomado utilizando el mismo intervalo y quitando los fines de semana, de esta manera la<BR>"
   print "<FONT SIZE=2><FONT FACE=Arial>   media se calcula solo utilizando datos recogidos en el intervalo productivo de la maquina.<BR>"
   print "</BODY>"
   print "</HTML>"
 }'
) | /usr/sbin/sendmail -f neteye-monitoring@wurth.es $PARA

}

# --------------------------------------------------------------------------- 
# START MAIN PROGRAM
# --------------------------------------------------------------------------- 

CPU=`$RRD_VALUES_CMD -H "$HOST" -s "$CPUSERVICE" -F "$FROMTIME" -T "$TOTIME"`
if [ $? -ne 0 ]
then
	echo "Error executing command: $RRD_VALUES_CMD -H '$HOST' -s '$CPUSERVICE' -F '$FROMTIME' -T '$TOTIME'"
	exit 2
fi
CPU_DY=`echo "$CPU" | cut -d\; -f1 | cut -d: -f2`
CPU_DB=`echo "$CPU" | cut -d\; -f2 | cut -d: -f2`
CPU_DL=`echo "$CPU" | cut -d\; -f3 | cut -d: -f2`
CPU_D7=`echo "$CPU" | cut -d\; -f4 | cut -d: -f2`
CPU_DM=`echo "$CPU" | cut -d\; -f5 | cut -d: -f2`

CPUPROCS=`$RRD_VALUES_CMD -H "$HOST" -s "$CPUPROCSSERVICE" -F "$FROMTIME" -T "$TOTIME"`
if [ $? -ne 0 ]
then
	echo "Error executing command: $RRD_VALUES_CMD -H '$HOST' -s '$CPUPROCSSERVICE' -F '$FROMTIME' -T '$TOTIME'"
	exit 2
fi
CPUPROCS_DY=`echo "$CPUPROCS" | cut -d\; -f1 | cut -d: -f2`
CPUPROCS_DB=`echo "$CPUPROCS" | cut -d\; -f2 | cut -d: -f2`
CPUPROCS_DL=`echo "$CPUPROCS" | cut -d\; -f3 | cut -d: -f2`
CPUPROCS_D7=`echo "$CPUPROCS" | cut -d\; -f4 | cut -d: -f2`
CPUPROCS_DM=`echo "$CPUPROCS" | cut -d\; -f5 | cut -d: -f2`

PROCS=`$RRD_VALUES_CMD -H "$HOST" -s "$PROCSSERVICE" -F "$FROMTIME" -T "$TOTIME"`
if [ $? -ne 0 ]
then
	echo "Error executing command: $RRD_VALUES_CMD -H '$HOST' -s '$PROCSSERVICE' -F '$FROMTIME' -T '$TOTIME'"
	exit 2
fi
PROCS_DY=`echo "$PROCS" | cut -d\; -f1 | cut -d: -f2`
PROCS_DB=`echo "$PROCS" | cut -d\; -f2 | cut -d: -f2`
PROCS_DL=`echo "$PROCS" | cut -d\; -f3 | cut -d: -f2`
PROCS_D7=`echo "$PROCS" | cut -d\; -f4 | cut -d: -f2`
PROCS_DM=`echo "$PROCS" | cut -d\; -f5 | cut -d: -f2`

SWAP=`$RRD_VALUES_CMD -H "$HOST" -s "$SWAPSERVICE" -F "$FROMTIME" -T "$TOTIME"`
if [ $? -ne 0 ]
then
	echo "Error executing command: $RRD_VALUES_CMD -H '$HOST' -s '$SWAPSERVICE' -F '$FROMTIME' -T '$TOTIME'"
	exit 2
fi
SWAP_DY=`echo "$SWAP" | cut -d\; -f1 | cut -d: -f2`
SWAP_DB=`echo "$SWAP" | cut -d\; -f2 | cut -d: -f2`
SWAP_DL=`echo "$SWAP" | cut -d\; -f3 | cut -d: -f2`
SWAP_D7=`echo "$SWAP" | cut -d\; -f4 | cut -d: -f2`
SWAP_DM=`echo "$SWAP" | cut -d\; -f5 | cut -d: -f2`

WIO=`$RRD_VALUES_CMD -H "$HOST" -s "$WIOSERVICE" -F "$FROMTIME" -T "$TOTIME"`
if [ $? -ne 0 ]
then
	echo "Error executing command: $RRD_VALUES_CMD -H '$HOST' -s '$WIOSERVICE' -F '$FROMTIME' -T '$TOTIME'"
	exit 2
fi
WIO_DY=`echo "$WIO" | cut -d\; -f1 | cut -d: -f2`
WIO_DB=`echo "$WIO" | cut -d\; -f2 | cut -d: -f2`
WIO_DL=`echo "$WIO" | cut -d\; -f3 | cut -d: -f2`
WIO_D7=`echo "$WIO" | cut -d\; -f4 | cut -d: -f2`
WIO_DM=`echo "$WIO" | cut -d\; -f5 | cut -d: -f2`

DIO=`$RRD_VALUES_CMD -H "$HOST" -s "$DIOSERVICE" -F "$FROMTIME" -T "$TOTIME"`
if [ $? -ne 0 ]
then
	echo "Error executing command: $RRD_VALUES_CMD -H '$HOST' -s '$DIOSERVICE' -F '$FROMTIME' -T '$TOTIME'"
	exit 2
fi
DIO_DY=`echo "$DIO" | cut -d\; -f1 | cut -d: -f2`
DIO_DB=`echo "$DIO" | cut -d\; -f2 | cut -d: -f2`
DIO_DL=`echo "$DIO" | cut -d\; -f3 | cut -d: -f2`
DIO_D7=`echo "$DIO" | cut -d\; -f4 | cut -d: -f2`
DIO_DM=`echo "$DIO" | cut -d\; -f5 | cut -d: -f2`

MEM=`$RRD_VALUES_CMD -H "$HOST" -s "$MEMSERVICE" -F "$FROMTIME" -T "$TOTIME" -d 2`
if [ $? -ne 0 ]
then
	echo "Error executing command: $RRD_VALUES_CMD -H '$HOST' -s '$MEMSERVICE' -F '$FROMTIME' -T '$TOTIME'"
	exit 2
fi
MEM_DY=`echo "$MEM" | cut -d\; -f1 | cut -d: -f2`
MEM_DB=`echo "$MEM" | cut -d\; -f2 | cut -d: -f2`
MEM_DL=`echo "$MEM" | cut -d\; -f3 | cut -d: -f2`
MEM_D7=`echo "$MEM" | cut -d\; -f4 | cut -d: -f2`
MEM_DM=`echo "$MEM" | cut -d\; -f5 | cut -d: -f2`

# Set Dates in format dd/mm/YYYY
tt=`echo "$MEM" | cut -d\; -f1 | cut -d: -f1`
DATE_DAY=`echo "<? echo date('d/m/Y',$tt) ?>" | php`
tt=`echo "$MEM" | cut -d\; -f2 | cut -d: -f1`
DATE_DAY_BEFORE=`echo "<? echo date('d/m/Y',$tt) ?>" | php`
tt=`echo "$MEM" | cut -d\; -f3 | cut -d: -f1`
DATE_DAY_LASTWEEK=`echo "<? echo date('d/m/Y',$tt) ?>" | php`

printf "%-25s %+5s %+10s %+10s %+8s %+10s %+10s %+9s %+7s\n" "% Utilizacion CPU        |" "$CPU_DY|" "$CPU_DB|" "$CPU_DL|" "$CPU_D7|" "$CPU_DM|" >>$TMPFILE
printf "%-25s %+5s %+10s %+10s %+8s %+10s %+10s %+9s %+7s\n" "Cola de procesos en CPU  |" "$CPUPROCS_DY|" "$CPUPROCS_DB|" "$CPUPROCS_DL|" "$CPUPROCS_D7|" "$CPUPROCS_DM|" >>$TMPFILE
printf "%-25s %+5s %+10s %+10s %+8s %+10s %+10s %+9s %+7s\n" "Procesos activos         |" "$PROCS_DY|" "$PROCS_DB|" "$PROCS_DL|" "$PROCS_D7|" "$PROCS_DM|" >>$TMPFILE
printf "%-25s %+5s %+10s %+10s %+8s %+10s %+10s %+9s %+7s\n" "Swaping                  |" "$SWAP_DY|" "$SWAP_DB|" "$SWAP_DL|" "$SWAP_D7|" "$SWAP_DM|" >>$TMPFILE
printf "%-25s %+5s %+10s %+10s %+8s %+10s %+10s %+9s %+7s\n" "CPU Wait I/O             |" "$WIO_DY|" "$WIO_DB|" "$WIO_DL|" "$WIO_D7|" "$WIO_DM|" >>$TMPFILE
printf "%-25s %+5s %+10s %+10s %+8s %+10s %+10s %+9s %+7s\n" "% Utilizacion Memoria    |" "$MEM_DY|" "$MEM_DB|" "$MEM_DL|" "$MEM_D7|" "$MEM_DM|" >>$TMPFILE
printf "%-25s %+5s %+10s %+10s %+8s %+10s %+10s %+9s %+7s\n" "I/O Disco por segundo    |" "$DIO_DY|" "$DIO_DB|" "$DIO_DL|" "$DIO_D7|" "$DIO_DM|" >>$TMPFILE
printf "%-25s %+5s %+10s %+10s %+8s %+10s %+10s %+9s %+7s\n" "Paquetes Red por segundo |" "$NET_DIA|" "$NET_ANT|" "$NET_DSA|" "$NET_7DI|" "$NET_30D|" >>$TMPFILE


send_html_email ITO $EMAIL $TMPFILE $HOST "De $FROMTIME a $TOTIME" $DATE_DAY $DATE_DAY_BEFORE $DATE_DAY_LASTWEEK 100

